import 'dart:async';
import 'package:http/http.dart';
import 'package:web3dart/web3dart.dart';

class Web3 {

  Web3Client _client;
  Future<bool> _approveCb;
  Credentials _credentials;

  Web3(String url, Future<bool> approveCb()) {
    _client = new Web3Client(url, new Client());
    _approveCb = approveCb();
  }

  Future<void> credentials(String pkey) async {
    _credentials = await _client.credentialsFromPrivateKey(pkey);
  }

  Future<String> sendTransactionAndWaitForReceipt(Transaction transaction) async {
    print('sendTransactionAndWaitForReceipt');    
    String txHash = await _client.sendTransaction(_credentials, transaction, chainId: await _client.getNetworkId());
    TransactionReceipt receipt = await _client.getTransactionReceipt(txHash);
    num delay = 1;
    num retries = 5;
    while(receipt == null) {
      print('waiting for receipt');
      await Future.delayed(new Duration(seconds: delay));
      delay*=2;
      retries--;
      if (retries == 0) {
        throw 'transaction $txHash not mined...';
      }
      receipt = await _client.getTransactionReceipt(txHash);
    }
    return txHash;
  }

  Future<String> transferNative(String _receiver, num _amountInEth) async {
    print('transferNative --> receiver: $_receiver, amount: $_amountInEth');

    bool isApproved = await _approveCb;
    if (!isApproved) {
      throw 'transaction not approved';
    }

    EthereumAddress receiver = EthereumAddress.fromHex(_receiver);
    EtherAmount amount = EtherAmount.fromUnitAndValue(EtherUnit.ether, BigInt.from(_amountInEth));

    String txHash = await sendTransactionAndWaitForReceipt(Transaction(
      to: receiver,
      value: amount
    ));
    print('transction $txHash successful');
    return txHash;
  }

  Future<EtherAmount> getBalance() async {
    EthereumAddress address = await _credentials.extractAddress();
    return _client.getBalance(address);
  }
}